unit Replace;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  BELib, StdCtrls, IniFiles, Menus;

type
  TfrmReplace = class(TForm)
    Label1: TLabel;
    cmbFindText: TComboBox;
    Label2: TLabel;
    cmbReplaceText: TComboBox;
    chkIgnoreCase: TCheckBox;
    btnReplace: TButton;
    btnCancel: TButton;
    btnFind: TButton;
    btnReplaceAll: TButton;
    procedure FormCreate(Sender: TObject);
    procedure btnFindClick(Sender: TObject);
    procedure btnReplaceClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnReplaceAllClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    procedure IniLoad;
    procedure IniSave;
    procedure AddFindText(AStr: String);
    procedure AddReplaceText(AStr: String);
    function DoFind(CellMove: Boolean): Integer;
    function DoReplace(All: Boolean): Integer;

  public
    { Public 錾 }
  end;

var
  frmReplace: TfrmReplace;

implementation

uses Main;

{$R *.DFM}

procedure TfrmReplace.FormCreate(Sender: TObject);
begin
  MoveMainFormCenter(Self, frmMain);
  IniLoad;
end;

procedure TfrmReplace.IniLoad;
var
  Ini: TIniFile;
  Sec: String;
  i: Integer;
  SL: TStringList;
begin
  Ini := TIniFile.Create(AppPath + IniName);
  try
    SL := TStringList.Create;
    try
      Sec := 'FindStrings';
      Ini.ReadSectionValues(Sec, SL);
      for i := 0 to SL.Count - 1 do
        cmbFindText.Items.Append(SL.Values[IntToHex(i, 1)]);

      SL.Clear;

      Sec := 'ReplaceStrings';
      Ini.ReadSectionValues(Sec, SL);
      for i := 0 to SL.Count - 1 do
        cmbReplaceText.Items.Append(SL.Values[IntToHex(i, 1)]);
    finally
      SL.Free;
    end;
    cmbFindText.ItemIndex := 0;
    cmbReplaceText.ItemIndex := 0;
 
  finally
    Ini.Free;
  end;
end;

procedure TfrmReplace.IniSave;
var
  Ini: TIniFile;
  Sec: String;
  i: Integer;
begin
  Ini := TIniFile.Create(AppPath + IniName);
  try
    Sec := 'FindStrings';
    for i := 0 to cmbFindText.Items.Count - 1 do
      Ini.WriteString(Sec, IntToHex(i, 1), '"' + cmbFindText.Items[i] + '"');
    Sec := 'ReplaceStrings';
    for i := 0 to cmbReplaceText.Items.Count - 1 do
      Ini.WriteString(Sec, IntToHex(i, 1), '"' + cmbReplaceText.Items[i] + '"');
  finally
    Ini.Free;
  end;
end;

procedure TfrmReplace.AddFindText(AStr: String);
var
  i: Integer;
begin
  if AStr = '' then Exit;
  cmbFindText.Items.Insert(0, AStr);
  for i := cmbFindText.Items.Count - 1 downto 1 do
    if cmbFindText.Items[i] = AStr then
      cmbFindText.Items.Delete(i);
  while cmbFindText.Items.Count > 16 do
      cmbFindText.Items.Delete(cmbFindText.Items.Count - 1);
  cmbFindText.ItemIndex := 0;
end;

procedure TfrmReplace.AddReplaceText(AStr: String);
var
  i: Integer;
begin
  cmbReplaceText.Items.Insert(0, AStr);
  for i := cmbReplaceText.Items.Count - 1 downto 1 do
    if cmbReplaceText.Items[i] = AStr then
      cmbReplaceText.Items.Delete(i);
  while cmbReplaceText.Items.Count > 16 do
      cmbReplaceText.Items.Delete(cmbReplaceText.Items.Count - 1);
  cmbReplaceText.ItemIndex := 0;
end;

function TfrmReplace.DoFind(CellMove: Boolean): Longint;
var
  SText: String;
begin
  Enabled := False; 
  LockForm(frmMain);
  try
    Result := -2;
    SText := cmbFindText.Text;
    if SText = '' then Exit;
    AddFindText(SText);
    frmMain.MxBinEditor1.SetSearchOptions(SText, chkIgnoreCase.Checked, False);
    Result := frmMain.MxBinEditor1.Find(SText, chkIgnoreCase.Checked, False, False, CellMove);
  finally
    UnLockForm(frmMain);
    Enabled := frmMain.Enabled; 
  end;
end;

function TfrmReplace.DoReplace(All: Boolean): Integer;
var
  RepCount, wPos: Integer;
begin
  Enabled := False;
  LockForm(frmMain);
  try

    if All then
      wPos := 0
    else
      wPos := frmMain.MxBinEditor1.CurrentAddress;

    AddFindText(cmbFindText.Text);
    AddReplaceText(cmbReplaceText.Text);
    frmMain.MxBinEditor1.SetSearchOptions(cmbFindText.Text, chkIgnoreCase.Checked, False);
    RepCount := frmMain.MxBinEditor1.Replace(wPos, cmbFindText.Text, cmbReplaceText.Text, All);

    if RepCount = 0 then
      MsgBox(cmbFindText.Text + ' ͌܂łB', MB_ICONINFORMATION)
    else if All then
    begin
      Hide;
      MsgBox(IntToStr(RepCount) + ' ̕ϊ܂B', MB_ICONINFORMATION);
    end;
    IniSave;
  finally
    UnLockForm(frmMain);
    Enabled := True; 
  end;
  Result := RepCount;
end;


procedure TfrmReplace.btnFindClick(Sender: TObject);
begin
  if DoFind(True) = -1 then
    MsgBox(cmbFindText.Text + ' ͌܂łB', MB_ICONINFORMATION);
end;

procedure TfrmReplace.btnReplaceClick(Sender: TObject);
begin
  DoReplace(False);
end;

procedure TfrmReplace.btnCancelClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmReplace.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TfrmReplace.btnReplaceAllClick(Sender: TObject);
begin
  if DoReplace(True) > 0 then Exit;
end;

procedure TfrmReplace.FormActivate(Sender: TObject);
begin
  frmMain.acOffsetJump.ShortCut   := ShortCut(0, []);
  frmMain.acUndo.ShortCut   := ShortCut(0, []);
  frmMain.acDelete.ShortCut := ShortCut(0, []);
  frmMain.acCopy.ShortCut   := ShortCut(0, []);
  frmMain.acCut.ShortCut    := ShortCut(0, []);
  frmMain.acPaste.ShortCut  := ShortCut(0, []);
end;

procedure TfrmReplace.FormDeactivate(Sender: TObject);
begin
  frmMain.acOffsetJump.ShortCut   := ShortCut(VK_F4, []);
  frmMain.acUndo.ShortCut   := ShortCut(Word('Z'), [ssCtrl]);
  frmMain.acDelete.ShortCut := ShortCut(VK_DELETE, []);
  frmMain.acCopy.ShortCut   := ShortCut(Word('C'), [ssCtrl]);
  frmMain.acCut.ShortCut    := ShortCut(Word('X'), [ssCtrl]);
  frmMain.acPaste.ShortCut  := ShortCut(Word('V'), [ssCtrl]);
end;

procedure TfrmReplace.FormDestroy(Sender: TObject);
begin
  frmReplace := nil;
end;

end.
